package com.huimv.manager.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.manager.dao.entity.ImagePathEntity;
import com.huimv.manager.dao.repo.ImagePathRepo;
import com.huimv.manager.service.IImageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class ImageServiceImpl implements IImageService {
    @Autowired
    private ImagePathRepo imagePathRepo;

    /**
     * @Method      : getLastImage
     * @Description : 读取最后一幅图像
     * @Params      : []
     * @Return      : java.lang.String
     *
     * @Author      : ZhuoNing
     * @Date        : 2021/1/21
     * @Time        : 20:49
     */
    @Override
    public ImagePathEntity getLastImage() {
        // 读取最后一幅图像
        ImagePathEntity imagePathEntity = imagePathRepo.getLastImage();
        return imagePathEntity;
    }

    /**
     * @Method      : getNextImage
     * @Description : 读取下一副图像
     * @Params      : []
     * @Return      : java.lang.String
     *
     * @Author      : ZhuoNing
     * @Date        : 2021/1/21
     * @Time        : 20:49
     */
    @Override
    public ImagePathEntity getNextImage() {
        // 读取下一副图像
        ImagePathEntity imagePathEntity = imagePathRepo.getNextImage();
        if(imagePathEntity!=null){
            // 更改记录
            imagePathEntity.setEffective(0);
            imagePathRepo.saveAndFlush(imagePathEntity);
            return imagePathEntity;
        }else{
            return imagePathRepo.getLastImage();
        }
    }

    /**
     * @Method      : getImageList
     * @Description : 
     * @Params      : [startDate, endDate, pageNo, pageSize]
     * @Return      : java.util.List<com.huimv.manager.dao.entity.ImagePathEntity>
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/22       
     * @Time        : 16:50
     */
    @Override
    public List<ImagePathEntity> getImageList(String startDate, String endDate,int pageNo,int pageSize) {
        System.out.println("pageNo>>"+pageNo);
        System.out.println("pageSize>>"+pageSize);
        int start = (pageNo-1)*pageSize;
        System.out.println("start>>"+start);
        // 按照日期区间，读取图片列表
        return imagePathRepo.listImage(startDate,endDate,start,pageSize);
    }

    /**
     * @Method      : getTotal
     * @Description : 
     * @Params      : [startDate, endDate]
     * @Return      : int
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/1/22       
     * @Time        : 16:50
     */
    @Override
    public int getTotal(String startDate, String endDate) {
        // 根据条件读取总记录数
        return imagePathRepo.getTotal(startDate,endDate);
    }

    @Override
    public List<ImagePathEntity> getExportImageList(String startDate, String endDate, String imageIds) {
        // 以指定图片导出优先条件
        if(imageIds == null || imageIds.trim().isEmpty()){
            // 用startDate,endDate读取需要导出的图片
            return imagePathRepo.getExportImageListByStartAndEndDate(startDate,endDate);
        }else{
            // 指定需要导出的图片
            String[] idArray = imageIds.split(",");
            List imageIdList = new ArrayList();
            for(String id:idArray){
                imageIdList.add(id);
            }
            // 读取导出图片
            return imagePathRepo.getExportImageListById(imageIdList);
        }
    }

    @Override
    public JSONObject getOneImage(String type, int imageId) {
        if(type.trim().equalsIgnoreCase("next")){
            // 读取下一张照片
            return  (JSONObject)JSONObject.toJSON(imagePathRepo.getNextImageById(imageId));
        }else{
            // 读取上一张照片
            return  (JSONObject)JSONObject.toJSON(imagePathRepo.getPreviousImageById(imageId));
        }
    }
}
